//+------------------------------------------------------------------+
//|                                               GoldHedgingEA.mq4 |
//|                        Copyright 2023, Your Name/Company Name |
//|                                         https://www.example.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Your Name/Company Name"
#property link      "https://www.example.com"
#property version   "1.00"
#property strict

//+------------------------------------------------------------------+
//| 重要提示 (Important Notes)                                       |
//+------------------------------------------------------------------+
//| 1. 这是一个基础的对冲EA框架。实现持续高盈利（如月30%）同时保持低回撤非常困难，需要复杂的策略和精密的风险管理。
//| 2. 此EA的默认参数仅为示例，【必须】经过在策略测试器中针对XAUUSD进行彻底的回测和优化才能用于实盘。
//| 3. 对冲策略本身并不能保证盈利，它旨在管理风险，但在某些市场条件下可能导致亏损累积。
//| 4. 务必在模拟账户上进行充分测试，了解其行为和风险。
//| 5. 高盈利目标通常伴随高风险，请谨慎评估。
//+------------------------------------------------------------------+

//--- 输入参数
input double LotSize      = 0.01; // 交易手数
input int    MaxSpread     = 30;   // 最大允许点差 (点)
input int    Slippage      = 3;    // 最大允许滑点 (点)
input int    MagicNumber   = 12345; // EA魔术数字
input string SymbolName    = "XAUUSD"; // 交易品种
input int    StopLossPoints  = 500;  // 止损点数 (例如 500 点 = 50 美元/手)
input int    TakeProfitPoints= 1000; // 止盈点数 (例如 1000 点 = 100 美元/手)
input double CloseNetProfit  = 20.0; // 平仓净盈利 (账户货币单位)
input double CloseNetLoss    = -10.0;// 平仓净亏损 (账户货币单位, 负数)
//--- 新增参数用于智能平仓
input int    SecureProfitTrailingStopPoints = 200; // (点) 盈利保护追踪止损距离 (0=禁用)
input int    SecureProfitMinProfitPoints    = 100; // (点) 激活盈利保护追踪止损的最小盈利点数
input int    RecoverLossCloseProfitPoints   = 100; // (点) 弥补亏损后关闭剩余订单的盈利点数 (0=禁用)

//--- 全局变量
int ticketBuy = 0;
int ticketSell = 0;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
//--- 初始化
   Print("Gold Hedging EA Initializing...");
   // 检查交易品种是否可用
   if(SymbolSelect(SymbolName, true) == false)
   {
      Print("Error selecting symbol: ", SymbolName);
      return(INIT_FAILED);
   }
   
   // 可以在这里添加更多的初始化检查，例如账户信息、服务器连接等
   Print("Gold Hedging EA Initialized Successfully.");
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
//--- 移除所有挂单和图表对象（如果需要）
   Print("Gold Hedging EA Deinitializing. Reason: ", reason);
//---   
}
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
{
//--- 检查交易条件是否满足
   if(!IsEATradeAllowed())
   {
      // Print("Trading not allowed."); // Optional: Uncomment if needed
      return;
   }
      
   if(Bars < 100) // 确保有足够的历史数据
   {
      Print("Waiting for sufficient historical data... Bars: ", Bars);
      return;
   }

   //--- 获取当前市场价格
   MqlTick current_tick;
   if(!SymbolInfoTick(SymbolName, current_tick))
   {
      Print("OnTick: Error getting tick for ", SymbolName, " Error code: ", GetLastError());
      return;
   }
   double ask = current_tick.ask;
   double bid = current_tick.bid;
   // Check for valid prices
   if (ask <= 0 || bid <= 0 || ask <= bid)
   {
       Print("OnTick: Invalid prices Ask=", ask, ", Bid=", bid);
       return;
   }
   double spread = (ask - bid) / Point; // 计算点差
   // Print("Tick: Ask=", ask, ", Bid=", bid, ", Spread=", spread);

   //--- 检查点差是否在允许范围内
   if(spread > MaxSpread)
   {
      // Print("Spread too high: ", spread, " points. Max allowed: ", MaxSpread);
      return; // 点差过大，暂时不交易
   }

   //--- 检查是否已有开仓订单
   bool hasOpenBuy = false;
   bool hasOpenSell = false;
   int openBuyCount = 0;
   int openSellCount = 0;
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
         if(OrderSymbol() == SymbolName && OrderMagicNumber() == MagicNumber)
         {
            if(OrderType() == OP_BUY)
            {
               hasOpenBuy = true;
               openBuyCount++;
            }
            if(OrderType() == OP_SELL)
            {
               hasOpenSell = true;
               openSellCount++;
            }
         }
      }
   }
   // Print("OnTick: hasOpenBuy=", hasOpenBuy, " (Count: ", openBuyCount, "), hasOpenSell=", hasOpenSell, " (Count: ", openSellCount, ")");

   //--- 核心对冲逻辑
   if(!hasOpenBuy && !hasOpenSell) // Case 1: 没有持仓 -> 开立初始对冲仓位
   {
      Print("OnTick: Case 1 - No open positions. Calling OpenHedgingPosition.");
      OpenHedgingPosition(ask, bid);
   }
   else if(hasOpenBuy && hasOpenSell) // Case 2: 双边持仓 -> 管理现有对冲仓位
   {
      // Print("OnTick: Case 2 - Both sides open. Calling ManageHedgingPositions.");
      ManageHedgingPositions(ask, bid);
   }
   else // Case 3: 只有单边持仓 -> 尝试智能管理或重新建立对冲
   {
      // Print("OnTick: Case 3 - Single side open. Calling ManageSinglePosition.");
      bool managed = ManageSinglePosition(hasOpenBuy, hasOpenSell, ask, bid);
      if(!managed) // 如果智能管理未执行操作（例如未达到条件或被禁用），则尝试重新对冲
      { 
         Print("OnTick: Case 3 - ManageSinglePosition returned false. Attempting re-hedge.");
         if(hasOpenBuy && !hasOpenSell)
         {
            Print("OnTick: Case 3 - Re-hedging with Sell.");
            // 计算止损止盈价格
            double sellStopLoss = bid + StopLossPoints * Point;
            double sellTakeProfit = bid - TakeProfitPoints * Point;
            // 开空单
            ticketSell = OrderSend(SymbolName, OP_SELL, LotSize, bid, Slippage, sellStopLoss, sellTakeProfit, "Re-Hedging Sell", MagicNumber, 0, clrRed);
            if(ticketSell < 0)
            {
               Print("OnTick: Error opening re-hedging sell order: ", GetLastError(), " (Symbol: ", SymbolName, ", Price: ", bid, ", SL: ", sellStopLoss, ", TP: ", sellTakeProfit, ")");
            }
            else
            {
               Print("OnTick: Opened Re-Hedging Sell Order #", ticketSell, " at ", bid, " SL: ", sellStopLoss, " TP: ", sellTakeProfit);
            }
         }
         else if(!hasOpenBuy && hasOpenSell)
         {
            Print("OnTick: Case 3 - Re-hedging with Buy.");
            // 计算止损止盈价格
            double buyStopLoss = ask - StopLossPoints * Point;
            double buyTakeProfit = ask + TakeProfitPoints * Point;
            // 开多单
            ticketBuy = OrderSend(SymbolName, OP_BUY, LotSize, ask, Slippage, buyStopLoss, buyTakeProfit, "Re-Hedging Buy", MagicNumber, 0, clrBlue);
            if(ticketBuy < 0)
            {
               Print("OnTick: Error opening re-hedging buy order: ", GetLastError(), " (Symbol: ", SymbolName, ", Price: ", ask, ", SL: ", buyStopLoss, ", TP: ", buyTakeProfit, ")");
            }
            else
            {
               Print("OnTick: Opened Re-Hedging Buy Order #", ticketBuy, " at ", ask, " SL: ", buyStopLoss, " TP: ", buyTakeProfit);
            }
         }
      }
      else
      {
         // Print("OnTick: Case 3 - ManageSinglePosition returned true (handled).");
      }
   }
   
   //--- 可以在这里添加其他交易逻辑，例如基于技术指标的入场/出场信号
   // Print("OnTick: End of tick processing.");

}
//+------------------------------------------------------------------+
//| Function to open initial hedging positions                       |
//+------------------------------------------------------------------+
void OpenHedgingPosition(double ask, double bid)
{
   Print("OpenHedgingPosition: Opening initial Buy and Sell. Ask=", ask, ", Bid=", bid);
   //--- 计算止损止盈价格
   double buyStopLoss = ask - StopLossPoints * Point;
   double buyTakeProfit = ask + TakeProfitPoints * Point;
   double sellStopLoss = bid + StopLossPoints * Point;
   double sellTakeProfit = bid - TakeProfitPoints * Point;

   //--- 开多单
   ticketBuy = OrderSend(SymbolName, OP_BUY, LotSize, ask, Slippage, buyStopLoss, buyTakeProfit, "Hedging Buy", MagicNumber, 0, clrBlue);
   if(ticketBuy < 0)
   {
      Print("OpenHedgingPosition: Error opening initial buy order: ", GetLastError(), " (Symbol: ", SymbolName, ", Price: ", ask, ", SL: ", buyStopLoss, ", TP: ", buyTakeProfit, ")");
   }
   else
   {
      Print("Opened Buy Order #", ticketBuy, " at ", ask, " SL: ", buyStopLoss, " TP: ", buyTakeProfit);
   }

   //--- 开空单
   ticketSell = OrderSend(SymbolName, OP_SELL, LotSize, bid, Slippage, sellStopLoss, sellTakeProfit, "Hedging Sell", MagicNumber, 0, clrRed);
   if(ticketSell < 0)
   {
      Print("OpenHedgingPosition: Error opening initial sell order: ", GetLastError(), " (Symbol: ", SymbolName, ", Price: ", bid, ", SL: ", sellStopLoss, ", TP: ", sellTakeProfit, ")");
   }
   else
   {
      Print("Opened Sell Order #", ticketSell, " at ", bid, " SL: ", sellStopLoss, " TP: ", sellTakeProfit);
   }
}

//+------------------------------------------------------------------+
//| Function to manage existing hedging positions                    |
//+------------------------------------------------------------------+
void ManageHedgingPositions(double ask, double bid)
{
   // Print("ManageHedgingPositions: Entered. Ask=", ask, ", Bid=", bid);
   //--- 计算当前EA所有持仓的总净盈亏 (包括手续费和隔夜利息)
   double totalNetProfit = 0;
   int openOrdersCount = 0;
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
         if(OrderSymbol() == SymbolName && OrderMagicNumber() == MagicNumber)
         {
            totalNetProfit += OrderProfit() + OrderSwap() + OrderCommission();
            openOrdersCount++;
         }
      }
   }

   //--- 如果只有单个方向的订单存在（可能一个被止损/止盈），则不应用对冲平仓逻辑
   // (这里可以添加更复杂的单边订单管理逻辑)
   if (openOrdersCount < 2) 
   {
       // Print("ManageHedgingPositions: Only one side exists (", openOrdersCount, "). Skipping net profit/loss check.");
       return; 
   }

   //--- 检查是否达到总盈利或总亏损目标
   if (CloseNetProfit > 0 && totalNetProfit >= CloseNetProfit)
   {
      Print("ManageHedgingPositions: Net Profit target reached (", totalNetProfit, " >= ", CloseNetProfit, "). Closing all positions.");
      CloseAllPositions(); // 达到总盈利目标，关闭所有仓位
   }
   else if (CloseNetLoss < 0 && totalNetProfit <= CloseNetLoss) // CloseNetLoss 是负数
   {
      Print("ManageHedgingPositions: Net Loss limit reached (", totalNetProfit, " <= ", CloseNetLoss, "). Closing all positions.");
      CloseAllPositions(); // 达到总亏损目标，关闭所有仓位
   }
   else
   {
      // Print("ManageHedgingPositions: No profit/loss target reached. totalNetProfit=", totalNetProfit);
   }
   
   //--- 注意：这里还可以添加更复杂的管理逻辑
   // 例如：
   // 1. 移动止损 (Trailing Stop)
   // 2. 根据技术指标（如均线、RSI）判断趋势，优先平掉逆势单
   // 3. 动态调整对冲比例（需要更复杂的订单管理）
}

//+------------------------------------------------------------------+
//| Function to close a specific position                            |
//+------------------------------------------------------------------+
bool CloseSpecificPosition(int ticket)
{
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
   {
      Print("Error selecting order #", ticket, " for closing: ", GetLastError());
      return(false);
   }

   if(OrderSymbol() != SymbolName || OrderMagicNumber() != MagicNumber)
   {
      // Not our order
      return(false); 
   }

   //--- Get current prices for closing
   MqlTick current_tick;
   if(!SymbolInfoTick(SymbolName, current_tick))
   {
      Print("Error getting tick for closing order #", ticket, ": ", GetLastError());
      // Attempt to refresh rates and try again (optional, can add complexity)
      RefreshRates();
      if(!SymbolInfoTick(SymbolName, current_tick))
      {
         Print("Failed to get tick even after RefreshRates() for closing order #", ticket);
         return(false); // Cannot get reliable price
      }
   }
   
   double close_price = 0;
   color arrow_color = clrNONE;
   int orderType = OrderType();
   
   if(orderType == OP_BUY)
   {
      close_price = current_tick.bid; // Use current bid to close buy
      arrow_color = clrRed;
   }
   else if(orderType == OP_SELL)
   {
      close_price = current_tick.ask; // Use current ask to close sell
      arrow_color = clrBlue;
   }
   else
   {
       Print("Order #", ticket, " has unknown type: ", orderType);
       return(false); // Should not happen for OP_BUY/OP_SELL
   }

   //--- Attempt to close the order
   bool closed = OrderClose(OrderTicket(), OrderLots(), close_price, Slippage, arrow_color);
   if(!closed)
   {
      Print("Error closing order #", OrderTicket(), " at price ", close_price, ". Error: ", GetLastError());
   }
   else
   {
      Print("Closed order #", OrderTicket(), " at price ", close_price);
   }
   return(closed);
}

//+------------------------------------------------------------------+
//| Function to close all positions for this EA                      |
//+------------------------------------------------------------------+
void CloseAllPositions()
{
   Print("Attempting to close all positions for MagicNumber: ", MagicNumber);
   int ticketsToClose[];
   int count = 0;
   
   // First, collect all relevant ticket numbers
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
         if(OrderSymbol() == SymbolName && OrderMagicNumber() == MagicNumber)
         {
            if(OrderType() == OP_BUY || OrderType() == OP_SELL)
            {
               ArrayResize(ticketsToClose, count + 1);
               ticketsToClose[count] = OrderTicket();
               count++;
            }
         }
      }
   }
   
   // Now, iterate through the collected tickets and close them
   Print("Found ", count, " positions to close.");
   for(int i = 0; i < count; i++)
   {
      int ticket = ticketsToClose[i];
      Print("Closing ticket #", ticket);
      // Use the improved CloseSpecificPosition function which gets current prices
      bool closed = CloseSpecificPosition(ticket); 
      if(closed)
      {
         // Optional: Short delay ONLY if experiencing issues with rapid closing attempts, 
         // but generally avoid Sleep() in OnTick context, especially for backtesting.
         // Sleep(50); 
      }
      else
      {
         // Error already printed in CloseSpecificPosition
         // Consider adding retry logic here if needed
      }
   }
   Print("Finished closing all positions attempt.");
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
bool ManageSinglePosition(bool hasBuy, bool hasSell, double ask, double bid)
{
   // 如果不是单边持仓状态，直接返回
   if (hasBuy == hasSell) 
   {
      // Print("ManageSinglePosition: Not a single position state (Buy=", hasBuy, ", Sell=", hasSell, "). Returning false.");
      return(false);
   }
   // Print("ManageSinglePosition: Entered. hasBuy=", hasBuy, ", hasSell=", hasSell, ", Ask=", ask, ", Bid=", bid);

   int ticket = -1;
   int orderType = -1;
   double openPrice = 0;
   double currentSL = 0;
   double currentTP = 0;
   double currentProfitPoints = 0;

   // 找到那个单独的订单
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
         if(OrderSymbol() == SymbolName && OrderMagicNumber() == MagicNumber)
         {
            ticket = OrderTicket();
            orderType = OrderType();
            openPrice = OrderOpenPrice();
            currentSL = OrderStopLoss();
            currentTP = OrderTakeProfit();
            break; // 假设只有一个符合条件的订单
         }
      }
   }

   if(ticket < 0) 
   {
      Print("ManageSinglePosition: Could not find the single order. Returning false.");
      return(false); // 未找到订单
   }
   // Print("ManageSinglePosition: Found single order #", ticket, ", Type=", orderType, ", OpenPrice=", openPrice);

   // 计算当前订单的盈利点数
   if(orderType == OP_BUY)
   {
      currentProfitPoints = (bid - openPrice) / Point;
   }
   else if(orderType == OP_SELL)
   {
      currentProfitPoints = (openPrice - ask) / Point;
   }
   // Print("ManageSinglePosition: Order #", ticket, " currentProfitPoints=", currentProfitPoints);

   // --- 逻辑 1: 尝试弥补亏损后平仓 (Recover Loss)
   if(RecoverLossCloseProfitPoints > 0 && currentProfitPoints >= RecoverLossCloseProfitPoints)
   {
      Print("ManageSinglePosition: RecoverLoss triggered for #", ticket, " (ProfitPts=", currentProfitPoints, " >= ", RecoverLossCloseProfitPoints, "). Attempting close.");
      // Use the improved function that gets its own price
      if(CloseSpecificPosition(ticket))
      {
         return(true); // 成功关闭
      }
      else
      {
         Print("ManageSinglePosition: RecoverLoss close failed for #", ticket);
         return(false); // 关闭失败
      }
   }

   // --- 逻辑 2: 尝试激活盈利保护追踪止损 (Secure Profit Trailing Stop)
   if(SecureProfitTrailingStopPoints > 0 && currentProfitPoints >= SecureProfitMinProfitPoints)
   {
      // Print("ManageSinglePosition: Checking Trailing Stop for #", ticket, " (ProfitPts=", currentProfitPoints, " >= ", SecureProfitMinProfitPoints, ")");
      double newSL = 0;
      bool modify = false;

      if(orderType == OP_BUY)
      {
         // 计算新的追踪止损位 (当前价格 - 追踪距离)
         newSL = bid - SecureProfitTrailingStopPoints * Point;
         // 只有当新止损位高于原止损位 (或原止损为0) 且高于开仓价时才修改
         if(newSL > currentSL && newSL > openPrice)
         {
            modify = true;
         }
      }
      else if(orderType == OP_SELL)
      {
         // 计算新的追踪止损位 (当前价格 + 追踪距离)
         newSL = ask + SecureProfitTrailingStopPoints * Point;
         // 只有当新止损位低于原止损位 (或原止损为0) 且低于开仓价时才修改
         if((newSL < currentSL || currentSL == 0) && newSL < openPrice)
         {
            modify = true;
         }
      }

      if(modify)
      {
         // 确保止损和止盈不冲突 (如果需要，可以取消止盈)
         // double newTP = (OrderTakeProfit() > 0) ? OrderTakeProfit() : 0;
         // if(orderType == OP_BUY && newSL >= newTP && newTP > 0) newTP = 0; // 如果SL超过TP，取消TP
         // if(orderType == OP_SELL && newSL <= newTP && newTP > 0) newTP = 0; // 如果SL超过TP，取消TP

         // 避免过于频繁的修改 - 修改条件：新SL有效且与当前SL不同
         if(newSL != currentSL) 
         {
             Print("Trailing Stop Activated/Updated for order #", ticket, ". New SL: ", newSL, " (Current Profit Pts: ", currentProfitPoints, ")");
             // 使用 OrderModify 修改止损位，保持止盈位不变
             if(OrderModify(ticket, openPrice, newSL, currentTP, 0, clrNONE))
             {
                 Print("ManageSinglePosition: Successfully modified SL for order #", ticket, " to ", newSL);
                 // 修改成功，可以认为已管理，阻止重新对冲
                 // 注意：这里返回 true 会阻止重新对冲。如果希望追踪止损激活后仍然允许未来某个时刻重新对冲（如果追踪止损没触发），则应返回 false。
                 // 暂时返回 true，表示已接管此订单的管理。
                 return(true); 
             }
             else
             {
                 Print("ManageSinglePosition: Error modifying SL for order #", ticket, ": ", GetLastError());
                 // 修改失败，不一定需要重新对冲，但记录错误
                 // return(false); // 保持原样或根据策略决定是否返回false
             }
         }
         else
         {
             // Print("ManageSinglePosition: New SL (", newSL, ") is the same as current SL (", currentSL, "). No modification needed.");
             // 即使没有修改，追踪逻辑也已激活，阻止重新对冲
             return(true);
         }
      }
      else
      {
          // 虽然达到最小盈利，但新SL不优于当前SL，继续持有，但阻止重新对冲
          return(true);
      }
   }

   // 如果以上条件都不满足，则返回 false，允许 OnTick 中的逻辑继续执行（例如重新对冲）
   return(false);
}

//+------------------------------------------------------------------+
//| Check if trading is allowed                                      |
//+------------------------------------------------------------------+
bool IsEATradeAllowed()
{
   // Check if connected to the trade server
   if(!IsConnected())
   {
      Print("IsTradeAllowed: Not connected to the trade server.");
      return(false);
   }
   
   // Check if trade is allowed for the account
   if(!AccountInfoInteger(ACCOUNT_TRADE_ALLOWED))
   {
      Print("IsTradeAllowed: Trading is not allowed for this account.");
      return(false);
   }
   
   // Check if expert advisor trading is allowed
   if(!AccountInfoInteger(ACCOUNT_TRADE_EXPERT))
   {
      Print("IsTradeAllowed: Expert Advisor trading is disabled in terminal settings.");
      return(false);
   }

   // Check if the trade context is busy (optional but recommended)
   if(IsTradeContextBusy())
   {
      Print("IsTradeAllowed: Trade context is busy. Retrying next tick.");
      return(false);
   }

   // Print("IsTradeAllowed: Trading is allowed."); // Optional: Uncomment for verbose logging
   return(true);
}
//+------------------------------------------------------------------+